#!/bin/bash
echo "Сборка сайта в двух помидорных темах и оптимизация результатов."
time_ms="$(date '+%s%3N')"
echo "Удаление каталогов предыдущей сборки."
rm -rf _site
rm -rf _site_older
rm -rf _site_color
echo "Сборка сайта в двух помидорных темах."
function jekyll_build {
  case "$1" in
  "older") echo "Сборка старого помидора." ;;
  "color") echo "Сборка цветного помидора." ;;
  *) return ;; # две помидорные темы оформления
  esac
  mkdir -p "_site_$1"
  cp -r "jekyll_site/ru" "_site_$1"
  cp -r "jekyll_site/en" "_site_$1"
  cp -r "jekyll_site/ru/index.md" "_site_$1"
  cp -r "jekyll_site/_config_$1.yml" "_site_$1/_config.yml"
  cp -r "jekyll_site/Gemfile_$1" "_site_$1/Gemfile"
  cd "_site_$1" || return
  jekyll build --disable-disk-cache --quiet
}
export -f jekyll_build
echo "Запуск параллельной сборки сайта в двух помидорных темах."
printf '%s\0' {older,color} | xargs -I{} -n1 -0 -P0 bash -c 'jekyll_build "{}"'
echo "Объединение двух сборок."
cp -r _site_older/_site .
cp -r _site_color/_site ./_site/color
echo "Копирование без сборки."
cp -r jekyll_site/img _site
cp -r jekyll_site/robots.txt _site
echo "Оптимизация собранного контента."
cd _site || exit
cp -r assets/* .
rm -r assets
rm -r color/assets/favicon.ico
cp -r color/assets/* .
rm -r color/assets
rm -r color/404.html
rm -r color/return.html
echo "Оптимизация ряда тегов."
function optimize_html {
  echo "Оптимизация: $1"
  sed -i 's|layout-padding=""|layout-padding|g' "$1"
  sed -i 's| class="language-plaintext highlighter-rouge"||g' "$1"
  sed -i 's| class="language-java highlighter-rouge"||g' "$1"
  sed -i 's|<div><div class="highlight">|<div class="highlight">|g' "$1"
  sed -i 's|</pre></div></div>|</pre></div>|g' "$1"
  sed -i 's|<hr />|<hr>|g' "$1"
  sed -i -r 's|<img(.+) />|<img\1>|g' "$1"
}
export -f optimize_html
echo "Запуск параллельной обработки собранных страниц и оптимизации ряда тегов."
find . -type f -name "*.html" -print0 | xargs -I{} -n1 -0 -P0 bash -c 'optimize_html "{}"'
echo "Страница перехода в корень сайта для каталогов без заглавной страницы."
find . -type d -print0 | xargs -I{} -n1 -0 -P0 cp -n return.html {}/index.html
rm -r return.html
echo "Общее время выполнения: $(("$(date '+%s%3N')" - "$time_ms")) мс."
